
/////////////////////////////////////////////////////////////
//
// Source file for BigBoxUI
//
//    This class implements the user interface created in 
//    RapidApp.
//
//    Restrict changes to those sections between
//    the "//--- Start/End editable code block" markers
//
//    This will allow RapidApp to integrate changes more easily
//
//    This class is a ViewKit user interface "component".
//    For more information on how components are used, see the
//    "ViewKit Programmers' Manual", and the RapidApp
//    User's Guide.
//
//
/////////////////////////////////////////////////////////////


#include "BigBoxUI.h" // Generated header file for this class

#include <Xm/BulletinB.h> 
#include <Xm/Scale.h> 
#include <Vk/VkResource.h>


// Externally defined classes referenced by this class: 

#include <Inventor/Xt/viewers/SoXtPlaneViewer.h>
#include <Inventor/So.h> // Includes ALL Inventor headers
                         // Replace for efficiency and faster compilation
//---- Start editable code block: headers and declarations


//---- End editable code block: headers and declarations



// These are default resources for widgets in objects of this class
// All resources will be prepended by *<name> at instantiation,
// where <name> is the name of the specific instance, as well as the
// name of the baseWidget. These are only defaults, and may be overriden
// in a resource file by providing a more specific resource name

String  BigBoxUI::_defaultBigBoxUIResources[] = {
        "*scale.titleString:  Interpolator",

        //---- Start editable code block: BigBoxUI Default Resources


        //---- End editable code block: BigBoxUI Default Resources

        (char*)NULL
};

BigBoxUI::BigBoxUI ( const char *name ) : VkComponent ( name ) 
{ 
    // No widgets are created by this constructor.
    // If an application creates a component using this constructor,
    // It must explictly call create at a later time.
    // This is mostly useful when adding pre-widget creation
    // code to a derived class constructor.

    //---- Start editable code block: BigBox constructor 2


    //---- End editable code block: BigBox constructor 2


}    // End Constructor




BigBoxUI::BigBoxUI ( const char *name, Widget parent ) : VkComponent ( name ) 
{ 
    //---- Start editable code block: BigBox pre-create


    //---- End editable code block: BigBox pre-create



    // Call creation function to build the widget tree.

     create ( parent );

    //---- Start editable code block: BigBox constructor


    //---- End editable code block: BigBox constructor


}    // End Constructor


BigBoxUI::~BigBoxUI() 
{
    delete _viewerFinal;

    delete _viewerMorphed;

    delete _viewerInitial;


    //---- Start editable code block: BigBoxUI destructor


    //---- End editable code block: BigBoxUI destructor
}    // End destructor



void BigBoxUI::create ( Widget parent )
{
    Arg      args[11];
    Cardinal count;
    count = 0;

    // Load any class-defaulted resources for this object

    setDefaultResources ( parent, _defaultBigBoxUIResources  );


    // Create an unmanaged widget as the top of the widget hierarchy

    _baseWidget = _bigBox = XtVaCreateWidget ( _name,
                                               xmBulletinBoardWidgetClass,
                                               parent, 
                                               XmNresizePolicy, XmRESIZE_GROW, 
                                               (XtPointer) NULL ); 

    // install a callback to guard against unexpected widget destruction

    installDestroyHandler();


    // Create widgets used in this component
    // All variables are data members of this class

    _scale = XtVaCreateManagedWidget  ( "scale",
                                         xmScaleWidgetClass,
                                         _baseWidget, 
                                         XmNmaximum, 1000, 
                                         XmNminimum, 0, 
                                         XmNorientation, XmHORIZONTAL, 
                                         XmNshowValue, True, 
                                         XmNdecimalPoints, 3, 
                                         XmNx, 340, 
                                         XmNy, 378, 
                                         XmNwidth, 300, 
                                         XmNheight, 70, 
                                         (XtPointer) NULL ); 

    XtAddCallback ( _scale,
                    XmNvalueChangedCallback,
                    &BigBoxUI::performMorphCallback,
                    (XtPointer) this ); 


    _viewerFinal = new SoXtPlaneViewer( _baseWidget, "viewerFinal");
    _viewerFinal->show();

    _viewerFinal->setDecoration(False);


    _viewerMorphed = new SoXtPlaneViewer( _baseWidget, "viewerMorphed");
    _viewerMorphed->show();

    _viewerMorphed->setDecoration(False);


    _viewerInitial = new SoXtPlaneViewer( _baseWidget, "viewerInitial");
    _viewerInitial->show();

    _viewerInitial->setDecoration(False);


    XtVaSetValues ( _viewerFinal->getWidget(),
                    XmNx, 650, 
                    XmNy, 10, 
                    XmNwidth, 320, 
                    XmNheight, 370, 
                    (XtPointer) NULL );
    XtVaSetValues ( _viewerMorphed->getWidget(),
                    XmNx, 330, 
                    XmNy, 10, 
                    XmNwidth, 320, 
                    XmNheight, 370, 
                    (XtPointer) NULL );
    XtVaSetValues ( _viewerInitial->getWidget(),
                    XmNx, 10, 
                    XmNy, 10, 
                    XmNwidth, 320, 
                    XmNheight, 370, 
                    (XtPointer) NULL );

    //---- Start editable code block: BigBoxUI create


    //---- End editable code block: BigBoxUI create
}

const char * BigBoxUI::className()
{
    return ("BigBoxUI");
}    // End className()




/////////////////////////////////////////////////////////////// 
// The following functions are static member functions used to 
// interface with Motif.
/////////////////////////////////// 

void BigBoxUI::performMorphCallback ( Widget    w,
                                      XtPointer clientData,
                                      XtPointer callData ) 
{ 
    BigBoxUI* obj = ( BigBoxUI * ) clientData;
    obj->performMorph ( w, callData );
}



/////////////////////////////////////////////////////////////// 
// The following functions are called from the menu items 
// in this window.
/////////////////////////////////// 

void BigBoxUI::performMorph ( Widget, XtPointer ) 
{
    // This virtual function is called from performMorphCallback.
    // This function is normally overriden by a derived class.

}



//---- Start editable code block: End of generated code


//---- End editable code block: End of generated code
