// In principle, for the morphing assingment you should not need to modify
// or look at this file. If you don't know Inventor, don't worry.
// Just use the functions provided in InventorStuff.h & morph.h


#include "InventorStuff.h"
#include <Inventor/So.h>  // includes ALL? Inventor header files
#include <Inventor/Xt/viewers/SoXtViewer.h>
#include <Inventor/SoPickedPoint.h>
#include <iostream.h>
#include "assert.h"
#include "morph.h"


#define NUM_WINDOWS 3

SoSeparator *bigRoot = NULL;
SoScale *imageSize = NULL;

SoNode *graph[NUM_WINDOWS];
SoLineSet *lines[NUM_WINDOWS];
SoPointSet *ptset[NUM_WINDOWS];
SoVertexProperty *points[NUM_WINDOWS];
SoTexture2 *textureImage[NUM_WINDOWS];
SoSeparator *imageGroup[NUM_WINDOWS];
SoSeparator *root[NUM_WINDOWS];
SoSeparator *markersGroup[NUM_WINDOWS];

SoOffscreenRenderer *renderer[2];
SoNode *warpgrid;

////////////////////////////////////////////////////////////////////
// function prototypes


int explicitEvent(SoEventCallback* eventCB,
		  int win, SbVec3f &imagePos,int& linePairNo, int &ptNo);

////////////////////////////////////////////////////////////////////
// this function is called by Inventor when an event (mouse press/move;key)
// happens over one of its windows

void myEventCB(void *w, SoEventCallback *eventCB){

    const SoEvent *event = eventCB->getEvent();


    // for debugging
    if (SO_KEY_PRESS_EVENT(event, W)) {
	cerr<<"Writing scene graph to morph.iv"<<endl;
	SoOutput o;
	o.openFile("morph.iv");
	SoWriteAction wa(&o);
	wa.apply(bigRoot);
	o.closeFile();
    }


    int win = (int)w;
    SbVec3f imgPos; int pairno, ptno;
    if(!explicitEvent(eventCB,win,imgPos,pairno,ptno)) return;

    ptno = ptno%2; // go from (global) point index to (line) endpoint index

    SbVec2s winPos = event->getPosition();


    if (SO_KEY_PRESS_EVENT(event, DELETE) || 
	SO_KEY_PRESS_EVENT(event, BACKSPACE) ) {
	delKey(win,winPos[0],winPos[1],imgPos[0],imgPos[1],pairno,ptno);
	eventCB->setHandled(); 
	return;
    }

    if(SO_MOUSE_PRESS_EVENT(event,BUTTON1)){
	mouseDown(1,win,winPos[0],winPos[1],imgPos[0],imgPos[1],pairno,ptno);
	eventCB->setHandled();
	eventCB->grabEvents();
	return;
    }
   
    if(SO_MOUSE_PRESS_EVENT(event,BUTTON2)){
	mouseDown(2,win,winPos[0],winPos[1],imgPos[0],imgPos[1],pairno,ptno);
	eventCB->setHandled();
	eventCB->grabEvents();
	return;
    }

    if(SO_MOUSE_RELEASE_EVENT(event,BUTTON1)){
	mouseUp(1,win,winPos[0],winPos[1],imgPos[0],imgPos[1],pairno,ptno);
	eventCB->setHandled();
	eventCB->releaseEvents();
	return;
    }

    if(SO_MOUSE_RELEASE_EVENT(event,BUTTON2)){
	mouseUp(2,win,winPos[0],winPos[1],imgPos[0],imgPos[1],pairno,ptno);
	eventCB->setHandled();
	eventCB->releaseEvents();
	return;
    }

    if( event->isOfType(SoLocation2Event::getClassTypeId())){
	mouseMove(win,winPos[0],winPos[1],imgPos[0],imgPos[1],pairno,ptno);
	eventCB->setHandled(); 
	return;
    }       
}

////////////////////////////////////////////////////////////////////

void initImageGraph(SoXtViewer *viewer,int win){
    static float square1[][3]= { {0,0,0}, {1,0,0}, {1,1,0},{0,1,0}};
    static float sq1tex[][2] = { {0,0}, {1,0}, {1,1}, {0,1} };
    static SoTranslation *tr;
    static SoFaceSet *fs = NULL;
    static SoDrawStyle *ds=NULL;

    // we don't need foreshortening; we'll look at the pictures from
    // straight above
    viewer->setCameraType(SoOrthographicCamera::getClassTypeId());
    // we want to see the picture when we move it
    viewer->setDrawStyle(SoXtViewer::INTERACTIVE,SoXtViewer::VIEW_AS_IS);

    // accumulate all graphs into a big one for debugging dump
    if(!bigRoot){
      bigRoot = new SoSeparator; // kept for debugging
      bigRoot->ref();
      tr = new SoTranslation;
      tr->ref(); tr->translation.setValue(SbVec3f(1.5,0,0));
    }

    // the root of the scene graph for this viewer
    root[win] = new SoSeparator;
    bigRoot->addChild(root[win]); bigRoot->addChild(tr);
    root[win]->renderCaching.setValue(SoSeparator::OFF);
    root[win]->setNumRenderCaches(0);
    root[win]->boundingBoxCaching.setValue(SoSeparator::OFF);
    root[win]->renderCulling.setValue(SoSeparator::OFF);
    root[win]->pickCulling.setValue(SoSeparator::OFF);
    viewer->setSceneGraph(root[win]);

    // center the square with the picture
    SoOrthographicCamera *camera = (SoOrthographicCamera*)viewer->getCamera();
    camera->height.setValue(1); camera->position.setValue(.5,.5,1);
    
    // this will set the displayed picture aspect ratio
    if(!imageSize) imageSize = new SoScale;
    root[win]->addChild(imageSize);

    markersGroup[win] = new SoSeparator;
    markersGroup[win]->ref();
    root[win]->addChild(markersGroup[win]);

    // add a node to draw the edited lines
    lines[win] = new SoLineSet; lines[win]->ref();
    markersGroup[win]->addChild(lines[win]);
    SoVertexProperty *vp = new SoVertexProperty;
    vp->materialBinding.setValue(SoVertexProperty::PER_PART);
    lines[win]->vertexProperty = vp;
    lines[win]->numVertices.setNum(0);

    // add a node to display the line endpoints ("arrowheads")
    if(!ds){ 
      ds = new SoDrawStyle;
      ds->ref();
      ds->pointSize.setValue(3); 
    }
    markersGroup[win]->addChild(ds);
    ptset[win] = new SoPointSet; ptset[win]->ref(); 
    markersGroup[win]->addChild(ptset[win]);
    points[win] = new SoVertexProperty;
    ptset[win]->vertexProperty.setValue( points[win] );
    //    ptset[win]->numPoints.setValue(0);
    points[win]->materialBinding.setValue(SoVertexProperty::PER_VERTEX);

    imageGroup[win] = new SoSeparator;
    imageGroup[win]->ref();
    root[win]->addChild( imageGroup[win] );

    // node to keep the image
    textureImage[win] = new SoTexture2;
    imageGroup[win]->addChild(textureImage[win]);

    // add a textured rectangle to display the image
    if(!fs){
      fs = new SoFaceSet;
      fs->ref();
      vp = new SoVertexProperty;
      fs->vertexProperty = vp;
      vp->vertex.setValues(0,4,square1);
      vp->texCoord.setValues(0,4,sq1tex);
      fs->numVertices.set1Value(0,4);
    }
    imageGroup[win]->addChild(fs);

    // an event callback node to allow interactive stuff
    SoEventCallback *ecb = new SoEventCallback; // handle keybd & mouse
    root[win]->addChild(ecb);
    ecb->addEventCallback(SoEvent::getClassTypeId(), 
		     myEventCB,
		     (void*)win );

    // this is needed for the RayPickAction. We need the viewer scene graph
    // (including the added camera)
    graph[win]=viewer->getSceneManager()->getSceneGraph();
}

////////////////////////////////////////////////////////////////////
void checkLines(int win){
    int n = morph::numLines(win);
    assert(ptset[win]->numPoints.getValue() == -1 ||
	   ptset[win]->numPoints.getValue() == 0 );
    assert(points[win]->vertex.getNum() == n);
    assert(points[win]->orderedRGBA.getNum() == n);
    assert(lines[win]->numVertices.getNum() == n || 
	   lines[win]->numVertices.getNum() == 0 );
    SoVertexProperty *vp = ((SoVertexProperty *)lines[win]
			    ->vertexProperty.getValue());
    assert( vp->vertex.getNum() == 2*n );
    assert( vp->orderedRGBA.getNum() == n );
    for(int i=0;i<n;i++) assert( lines[win]->numVertices[i]==2 );
}
////////////////////////////////////////////////////////////////////

void hideLines(int win){
    lines[win]->numVertices.setNum(0);
    ptset[win]->numPoints.setValue(0);
}

////////////////////////////////////////////////////////////////////

void showLines(int win){
    int n = morph::numLines(win);
    ptset[win]->numPoints.setValue(-1);
    lines[win]->numVertices.setNum(n);
    for(int i=0;i<n;i++)
      lines[win]->numVertices.set1Value(i,2);
    checkLines(win);
}


/////////////////////////////////////////////////////////////////////

void morph::setLineColor(int win, int n, uchar r, uchar g, uchar b){
    uint32_t color = ((uint32_t)r<<24) + ((uint32_t)g<<16) +
      ((uint32_t)b<<8) + 0xFF;
    SoVertexProperty *vp = ((SoVertexProperty*)lines[win]->
			    vertexProperty.getValue());
    if( n < vp->orderedRGBA.getNum() )
	vp->orderedRGBA.set1Value(n,color);
    if( n < points[win]->orderedRGBA.getNum() )
	points[win]->orderedRGBA.set1Value(n,color);
    
}


/////////////////////////////////////////////////////////////////////

void morph::getLine(int win, int n, float& x1, float& y1,
		    float& x2, float& y2){
    n *= 2;
    SoVertexProperty *vp = (SoVertexProperty *)
	lines[win]->vertexProperty.getValue();
    x1 = vp->vertex[n][0]; y1 = vp->vertex[n][1];
    x2 = vp->vertex[n+1][0]; y2 = vp->vertex[n+1][1];
}


/////////////////////////////////////////////////////////////////////


void morph::deleteLastLine(int win){
    int n = numLines(win) - 1;
    if(n<0) return;
    deleteLine(win,n);
}

/////////////////////////////////////////////////////////////////////

void morph::deleteLine(int win, int n){
    checkLines(win);
    if( n<0 || n>= numLines(win) ) return;

    int ind = root[win]->findChild( markersGroup[win] );
    root[win]->removeChild( markersGroup[win] );

    points[win]->vertex.deleteValues(n,1);
    points[win]->orderedRGBA.deleteValues(n,1);
    
    if( ptset[win]->numPoints.getValue() ) // lines not hidden
	lines[win]->numVertices.deleteValues(n,1);

    SoVertexProperty *vp = (SoVertexProperty *)
	lines[win]->vertexProperty.getValue();
    vp->vertex.deleteValues(n*2,2);
    vp->orderedRGBA.deleteValues(n,1);

    root[win]->insertChild(markersGroup[win],ind);
    checkLines(win);
}

/////////////////////////////////////////////////////////////////////

void morph::newLine(int win){
    checkLines(win);
    int n = numLines(win); 
    points[win]->vertex.setNum(n+1);
    points[win]->vertex.set1Value(n,0,0,0);
    points[win]->orderedRGBA.set1Value(n,0xFFFFFFFF);
    
    SoVertexProperty *vp = (SoVertexProperty *)
	lines[win]->vertexProperty.getValue();
    vp->vertex.setNum(2*n+2);
    vp->vertex.set1Value(2*n,0,0,0);
    vp->vertex.set1Value(2*n+1,0,0,0);
    vp->orderedRGBA.setNum(n+1);
    vp->orderedRGBA.set1Value(n,0xFFFFFFFF);
    
    if( ptset[win]->numPoints.getValue() ) // lines not hidden
	lines[win]->numVertices.set1Value(n,2);

    checkLines(win);
}

////////////////////////////////////////////////////////////////////

int morph::numLines(int win){
    return points[win]->vertex.getNum();
}
////////////////////////////////////////////////////////////////////

void morph::setLine(int win, int n, float x1, float y1, float x2, float y2){
    int n1 = numLines(win);
    assert( n>=0 && n<n1 );

    points[win]->vertex.set1Value(n,x2,y2,0.001);
    n *=2;
    SoVertexProperty *vp =(SoVertexProperty *)
	lines[win]->vertexProperty.getValue();
    vp->vertex.set1Value(n,x1,y1,.001);
    vp->vertex.set1Value(n+1,x2,y2,.001);
    checkLines(win);
}
////////////////////////////////////////////////////////////////////

void displayImage(int win, // 0 = initial, 1 = morph, 2 = final
		  int width, int height,
		  unsigned char *imgData){
    int i = root[win]->findChild( imageGroup[win] );
    root[win]->removeChild( imageGroup[win] );
    SbVec2s size( width, height );
    textureImage[win]->image.setValue(size,3,imgData);
    root[win]->insertChild( imageGroup[win], i );
}
////////////////////////////////////////////////////////////////////

const unsigned char* readImage_RGB(int window, // 0 = initial, 2 = final
			     const char *filename,
			     int &width, int &height){
    textureImage[window]->filename.setValue(filename);
    int nc;SbVec2s s;
    const unsigned char* imgdata = textureImage[window]->image.getValue(s,nc);
    width=s[0]; height=s[1];

    if( filename[0] ) 
	imageSize->scaleFactor.setValue((float)width/height,1,1);
    return imgdata;
}

////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////


float ptSegmentDist(SbVec3f p, SbVec3f a, SbVec3f b){
    SbVec3f pab = SbLine(a,b).getClosestPoint(p);
    SbBox3f bb; bb.makeEmpty(); bb.extendBy(a); bb.extendBy(b);
    if( bb.intersect(pab) ) return (p-pab).length();
    float d = (a-p).length(), d1 =  (b-p).length();
    return d<d1?d:d1;
}


////////////////////////////////////////////////////////////////////

int findClosestSegment(int win, SbVec3f p, float &dist){
    int closest = -1;
    SoVertexProperty *vp = (SoVertexProperty *)
	lines[win]->vertexProperty.getValue();
    dist = 1e31;
    for(int i=0;i<vp->vertex.getNum();i+=2){
	float d = ptSegmentDist(p, vp->vertex[i], vp->vertex[i+1]);
	if( d<dist ){ dist = d; closest = i/2; }
    }
    return closest;
} 
////////////////////////////////////////////////////////////////////

int findClosestPt(int win, SbVec3f p, float &dist){
    int closest = -1;
    SoVertexProperty *vp = (SoVertexProperty *)
	lines[win]->vertexProperty.getValue();
    dist = 1e31;
    for(int i=0;i<vp->vertex.getNum();i++){
	float d = (p - vp->vertex[i]).length();
	if( d<dist ){ dist = d; closest = i; }
    }
    return closest;
} 

////////////////////////////////////////////////////////////////////

int explicitEvent(SoEventCallback *eventCB,
		  int win, SbVec3f &imagePos,int& linePairNo, int& ptNo){

    imagePos = SbVec3f(-1,-1,-1); linePairNo = -1; ptNo = -1;

    int foundLine=0;
    
    const SbViewportRegion &myRegion = 
	eventCB->getAction()->getViewportRegion();
    SoRayPickAction myPickAction( myRegion );
    myPickAction.setRadius(3);
    myPickAction.setPoint( eventCB->getEvent()->getPosition( myRegion ) );
    myPickAction.setPickAll( TRUE );
    myPickAction.apply( graph[win] );
       
    SoPickedPointList myPickedPointList = myPickAction.getPickedPointList();
    if (myPickedPointList == NULL) return 0;

    for(int i=0; i < myPickedPointList.getLength(); i++ ){
	if( myPickedPointList[i]->getPath()->getTail()->
	    isOfType( SoFaceSet::getClassTypeId() ) ) 
	    imagePos = myPickedPointList[i]->getObjectPoint();

	if( myPickedPointList[i]->getPath()->getTail()->
	    isOfType( SoLineSet::getClassTypeId() ) ){
	    foundLine=1;
	    /* For some reason, Details don't seem to work.
	       Maybe bacuse I used VertexProperty instead of Coordinate3?
	       oh well...
	    SoLineDetail *d = (SoLineDetail*)myPickedPointList[i]->getDetail
	      (myPickedPointList[i]->getPath()->getTail());
	    if(d) linePairNo = d->getLineIndex();
	    else{
	      cerr<<"no detail associated with the line"<<endl;
	      SoWriteAction wa;
	      wa.apply(myPickedPointList[i]->getPath());
	    }
	    */
	}
    }
    
    if(foundLine){
	    // here's the old fashioned way:
	    float dist;
	    linePairNo = findClosestSegment(win,imagePos,dist);
	    ptNo = findClosestPt(win,imagePos,dist);
    }

    return (imagePos[0]>=0);
}

//////////////////////////////////////////////////////////////////////////////

uchar* inventorWarp(int gridsize, int fromWin,t_mapfn mapfn, 
		    int width, int height,
		    int rendererNo){
    static int first=1;
    static SoSeparator *root;
    static SoVertexProperty *vp;
    static SoIndexedFaceSet *fs;
    static SoOrthographicCamera *camera;
    static int oldgs = -1;

    if( first ){
	first=0;
	fs = new SoIndexedFaceSet; fs->ref();
	vp = new SoVertexProperty; fs->vertexProperty.setValue(vp);
	warpgrid = fs;
	renderer[0] = renderer[1] = NULL;
    }

    int newgs = (gridsize!=oldgs);
    oldgs = gridsize;

    vp->vertex.setNum((gridsize+1)*(gridsize+1));
    vp->texCoord.setNum((gridsize+1)*(gridsize+1));
    if( newgs ) fs->coordIndex.setNum(5*gridsize*gridsize);

    int i,j,k=0,l=0;
    for(i=0;i<=gridsize;i++)
	for(j=0;j<=gridsize;j++){
	    float x = (float)i/gridsize, y= (float)j/gridsize, xm,ym;
	    mapfn(x,y, fromWin,1, xm,ym);
	    
	    vp->vertex.set1Value(k,xm,ym,0.001);
	    if ( newgs ){
		vp->texCoord.set1Value(k,x,y);

		if(i&&j){
		    fs->coordIndex.set1Value(l++,k);
		    fs->coordIndex.set1Value(l++,k-1);
		    fs->coordIndex.set1Value(l++,k-gridsize-2);
		    fs->coordIndex.set1Value(l++,k-gridsize-1);
		    fs->coordIndex.set1Value(l++,-1);
		}
	    }
	    k++;
	}   

    return 0;
}

/////////////////////////////////////////////////////////////////////////////

void toggleLastWarpGrid(int win){
static SoDrawStyle *ds=0;
    SoSearchAction sa;

    if(!ds){
	ds = new SoDrawStyle;
	ds->style.setValue(SoDrawStyle::LINES);
	ds->ref();
    }

    sa.setInterest(SoSearchAction::FIRST);
    sa.setNode(warpgrid);
    sa.apply(graph[win]);
    SoPath *p = sa.getPath();
    if( p ){ 
	SoSeparator *s = (SoSeparator*)p->getNode(p->getLength()-2);
	assert(s->findChild(ds)>=0);
	s->removeChild(ds);
	assert(s->findChild(warpgrid)>=0);
	s->removeChild(warpgrid);
    }
    else {
	sa.reset();
	sa.setNode(imageGroup[win]);
	sa.setInterest(SoSearchAction::FIRST);
	sa.apply(graph[win]);
	SoPath *p = sa.getPath();
	if(p){ 
	    SoSeparator *s = (SoSeparator*)p->getNode(p->getLength()-2);
	    // insert before event callback node
	    s->insertChild(ds,s->getNumChildren()-1);
	    s->insertChild(warpgrid,s->getNumChildren()-1);
	}
    }

}
