
//////////////////////////////////////////////////////////////
//
// Source file for mainWindow
//
//    This class is a subclass of VkWindow
//
//
// Normally, very little in this file should need to be changed.
// Create/add/modify menus using RapidApp.
//
// Try to restrict any changes to the bodies of functions
// corresponding to menu items, the constructor and destructor.
//
// Restrict changes to those sections between
// the "//--- Start/End editable code block" markers
//
// Doing so will allow you to make changes using RapidApp
// without losing any changes you may have made manually
//
// Avoid gratuitous reformatting and other changes that might
// make it difficult to integrate changes made using RapidApp
//////////////////////////////////////////////////////////////
#include "mainWindow.h"

#include <Vk/VkApp.h>
#include <Vk/VkFileSelectionDialog.h>
#include <Vk/VkSubMenu.h>
#include <Vk/VkRadioSubMenu.h>
#include <Vk/VkMenuItem.h>
#include <Vk/VkMenuBar.h>
#include <Vk/VkResource.h>


// Externally defined classes referenced by this class: 

#include "BigBox.h"

extern void VkUnimplemented ( Widget, const char * );

//---- Start editable code block: headers and declarations

#include "morph.h"
#include <Vk/VkErrorDialog.h>

//---- End editable code block: headers and declarations



// These are default resources for widgets in objects of this class
// All resources will be prepended by *<name> at instantiation,
// where <name> is the name of the specific instance, as well as the
// name of the baseWidget. These are only defaults, and may be overriden
// in a resource file by providing a more specific resource name

String  mainWindow::_defaultmainWindowResources[] = {
        "*defaultClassName:  mainWindow",
        "*exitButton.accelerator:  Ctrl<Key>Q",
        "*exitButton.acceleratorText:  Ctrl+Q",
        "*exitButton.labelString:  Exit",
        "*exitButton.mnemonic:  x",
        "*filePane.labelString:  File",
        "*filePane.mnemonic:  F",
        "*finalImg.accelerator:  Ctrl<Key>F",
        "*finalImg.acceleratorText:  Ctrl+F",
        "*finalImg.labelString:  Final Image",
        "*helpPane.labelString:  Help",
        "*helpPane.mnemonic:  H",
        "*help_click_for_help.accelerator:  Shift<Key>F1",
        "*help_click_for_help.acceleratorText:  Shift+F1",
        "*help_click_for_help.labelString:  Click For Help",
        "*help_click_for_help.mnemonic:  C",
        "*help_index.labelString:  Index",
        "*help_index.mnemonic:  I",
        "*help_keys_and_short.labelString:  Keys and Shortcuts",
        "*help_keys_and_short.mnemonic:  K",
        "*help_overview.labelString:  Overview",
        "*help_overview.mnemonic:  O",
        "*help_prod_info.labelString:  Product Information",
        "*help_prod_info.mnemonic:  P",
        "*initialImg.accelerator:  Ctrl<Key>I",
        "*initialImg.acceleratorText:  Ctrl+I",
        "*initialImg.labelString:  Initial Image",
        "*interpLines.labelString:  Interp Lines",
        "*morphed.labelString:  Morphed",
        "*new1.accelerator:  Ctrl<Key>N",
        "*new1.acceleratorText:  Ctrl+N",
        "*new1.labelString:  New",
        "*new1.mnemonic:  N",
        "*newButton.accelerator:  Ctrl<Key>O",
        "*newButton.acceleratorText:  Ctrl+O",
        "*newButton.labelString:  Open...",
        "*newButton.mnemonic:  O",
        "*openButton.accelerator:  Ctrl<Key>S",
        "*openButton.acceleratorText:  Ctrl+S",
        "*openButton.labelString:  Save",
        "*openButton.mnemonic:  S",
        "*saveButton.accelerator:  Ctrl<Key>A",
        "*saveButton.acceleratorText:  Ctrl+A",
        "*saveButton.labelString:  save As",
        "*saveButton.mnemonic:  A",
        "*saveFrame.labelString:  Save Frame",
        "*saveMovie.labelString:  Save Movie",
        "*show.labelString:  Show",
        "*title:  Image Morph   - COS 420 / fall 98",
        "*view.labelString:  View",
        "*warpedFinal.labelString:  Warped Final",
        "*warpedInitial.labelString:  Warped Initial",

        //---- Start editable code block: mainWindow Default Resources


        //---- End editable code block: mainWindow Default Resources

        (char*)NULL
};


//---- Class declaration

mainWindow::mainWindow ( const char *name,
                        ArgList args,
                        Cardinal argCount) : 
                  VkWindow ( name, args, argCount )
{
    // Load any class-default resources for this object

    setDefaultResources ( baseWidget(), _defaultmainWindowResources  );


    // Create the view component contained by this window

    _bigBox = new BigBox ( "bigBox",mainWindowWidget() );


    XtVaSetValues ( _bigBox->baseWidget(),
                    XmNwidth, 984, 
                    XmNheight, 458, 
                    (XtPointer) NULL );

    // Add the component as the main view

    addView ( _bigBox );

    // Create the panes of this window's menubar. The menubar itself
    // is created automatically by ViewKit


    // The filePane menu pane

    _filePane =  addMenuPane ( "filePane" );

    _new1 =  _filePane->addAction ( "new1", 
                                        &mainWindow::doNewCallback, 
                                        (XtPointer) this );

    _newButton =  _filePane->addAction ( "newButton", 
                                        &mainWindow::openFileCallback, 
                                        (XtPointer) this );

    _openButton =  _filePane->addAction ( "openButton", 
                                        &mainWindow::saveCallback, 
                                        (XtPointer) this );

    _saveButton =  _filePane->addAction ( "saveButton", 
                                        &mainWindow::saveasCallback, 
                                        (XtPointer) this );

    _separator1 =  _filePane->addSeparator();

    _initialImg =  _filePane->addAction ( "initialImg", 
                                        &mainWindow::doInitialImgCallback, 
                                        (XtPointer) this );

    _finalImg =  _filePane->addAction ( "finalImg", 
                                        &mainWindow::doFinalImgCallback, 
                                        (XtPointer) this );

    _saveFrame =  _filePane->addAction ( "saveFrame", 
                                        &mainWindow::doSaveFrameCallback, 
                                        (XtPointer) this );

    _saveMovie =  _filePane->addAction ( "saveMovie", 
                                        &mainWindow::doSaveMovieCallback, 
                                        (XtPointer) this );

    _separator2 =  _filePane->addSeparator();

    _exitButton =  _filePane->addAction ( "exitButton", 
                                        &mainWindow::quitCallback, 
                                        (XtPointer) this );

    // The view menu pane

    _view =  addMenuPane ( "view" );

    // The show menu pane

    _show =  _view->addRadioSubmenu ( "show" );

    _warpedInitial =  _show->addToggle ( "warpedInitial", 
                                        &mainWindow::setWarpedInitialCallback, 
                                        (XtPointer) this );

    _morphed =  _show->addToggle ( "morphed", 
                                        &mainWindow::setMorphedCallback, 
                                        (XtPointer) this );

    _warpedFinal =  _show->addToggle ( "warpedFinal", 
                                        &mainWindow::setWarpedFinalCallback, 
                                        (XtPointer) this );

    _interpLines =  _view->addToggle ( "interpLines", 
                                        &mainWindow::setInterpLinesCallback, 
                                        (XtPointer) this );


    //---- Start editable code block: mainWindow constructor


    //---- End editable code block: mainWindow constructor


}    // End Constructor


mainWindow::~mainWindow()
{
    delete _bigBox;
    //---- Start editable code block: mainWindow destructor


    //---- End editable code block: mainWindow destructor
}    // End destructor


const char *mainWindow::className()
{
    return ("mainWindow");
}    // End className()


Boolean mainWindow::okToQuit()
{
    //---- Start editable code block: mainWindow okToQuit



    // This member function is called when the user quits by calling
    // theApplication->terminate() or uses the window manager close protocol
    // This function can abort the operation by returning FALSE, or do some.
    // cleanup before returning TRUE. The actual decision is normally passed on
    // to the view object



    return ( TRUE );

    //---- End editable code block: mainWindow okToQuit
}    // End okToQuit()

void mainWindow::afterRealizeHook()
{
    // Retrieve the help menu pane and remove it

    menu()->showHelpPane(FALSE);
}



/////////////////////////////////////////////////////////////// 
// The following functions are static member functions used to 
// interface with Motif.
/////////////////////////////////// 

void mainWindow::doFinalImgCallback ( Widget    w,
                                      XtPointer clientData,
                                      XtPointer callData ) 
{ 
    mainWindow* obj = ( mainWindow * ) clientData;
    obj->doFinalImg ( w, callData );
}

void mainWindow::doInitialImgCallback ( Widget    w,
                                        XtPointer clientData,
                                        XtPointer callData ) 
{ 
    mainWindow* obj = ( mainWindow * ) clientData;
    obj->doInitialImg ( w, callData );
}

void mainWindow::doNewCallback ( Widget    w,
                                 XtPointer clientData,
                                 XtPointer callData ) 
{ 
    mainWindow* obj = ( mainWindow * ) clientData;
    obj->doNew ( w, callData );
}

void mainWindow::doSaveFrameCallback ( Widget    w,
                                       XtPointer clientData,
                                       XtPointer callData ) 
{ 
    mainWindow* obj = ( mainWindow * ) clientData;
    obj->doSaveFrame ( w, callData );
}

void mainWindow::doSaveMovieCallback ( Widget    w,
                                       XtPointer clientData,
                                       XtPointer callData ) 
{ 
    mainWindow* obj = ( mainWindow * ) clientData;
    obj->doSaveMovie ( w, callData );
}

void mainWindow::openFileCallback ( Widget    w,
                                    XtPointer clientData,
                                    XtPointer callData ) 
{ 
    mainWindow* obj = ( mainWindow * ) clientData;
    obj->openFile ( w, callData );
}

void mainWindow::quitCallback ( Widget    w,
                                XtPointer clientData,
                                XtPointer callData ) 
{ 
    mainWindow* obj = ( mainWindow * ) clientData;
    obj->quit ( w, callData );
}

void mainWindow::saveCallback ( Widget    w,
                                XtPointer clientData,
                                XtPointer callData ) 
{ 
    mainWindow* obj = ( mainWindow * ) clientData;
    obj->save ( w, callData );
}

void mainWindow::saveasCallback ( Widget    w,
                                  XtPointer clientData,
                                  XtPointer callData ) 
{ 
    mainWindow* obj = ( mainWindow * ) clientData;
    obj->saveas ( w, callData );
}

void mainWindow::setInterpLinesCallback ( Widget    w,
                                          XtPointer clientData,
                                          XtPointer callData ) 
{ 
    mainWindow* obj = ( mainWindow * ) clientData;
    obj->setInterpLines ( w, callData );
}

void mainWindow::setMorphedCallback ( Widget    w,
                                      XtPointer clientData,
                                      XtPointer callData ) 
{ 
    mainWindow* obj = ( mainWindow * ) clientData;
    obj->setMorphed ( w, callData );
}

void mainWindow::setWarpedFinalCallback ( Widget    w,
                                          XtPointer clientData,
                                          XtPointer callData ) 
{ 
    mainWindow* obj = ( mainWindow * ) clientData;
    obj->setWarpedFinal ( w, callData );
}

void mainWindow::setWarpedInitialCallback ( Widget    w,
                                            XtPointer clientData,
                                            XtPointer callData ) 
{ 
    mainWindow* obj = ( mainWindow * ) clientData;
    obj->setWarpedInitial ( w, callData );
}





/////////////////////////////////////////////////////////////// 
// The following functions are called from callbacks 
/////////////////////////////////// 

void mainWindow::doFinalImg ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: mainWindow doFinalImg

    char *err;
    theFileSelectionDialog->setFilterPattern("*.rgb");
    if(theFileSelectionDialog->postAndWait() == VkDialogManager::OK)
	err = morph::loadFinalImg(theFileSelectionDialog->fileName());
    if(err) theErrorDialog->post(err);

    //---- End editable code block: mainWindow doFinalImg

}    // End mainWindow::doFinalImg()


void mainWindow::doInitialImg ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: mainWindow doInitialImg

    char *err;
    theFileSelectionDialog->setFilterPattern("*.rgb");
    if(theFileSelectionDialog->postAndWait() == VkDialogManager::OK)
	err = morph::loadInitialImg(theFileSelectionDialog->fileName());
    if(err) theErrorDialog->post(err);

    //---- End editable code block: mainWindow doInitialImg

}    // End mainWindow::doInitialImg()


void mainWindow::doNew ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: mainWindow doNew

    morph::nnew();

    //---- End editable code block: mainWindow doNew

}    // End mainWindow::doNew()


void mainWindow::doSaveFrame ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: mainWindow doSaveFrame

    char *err=0;

    theFileSelectionDialog->setFilterPattern("*.ppm");
    if(theFileSelectionDialog->postAndWait() == VkDialogManager::OK)
        err = morph::saveFrame(theFileSelectionDialog->fileName());

    if( err ) theErrorDialog->post(err);

    //---- End editable code block: mainWindow doSaveFrame

}    // End mainWindow::doSaveFrame()


void mainWindow::doSaveMovie ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: mainWindow doSaveMovie

    char *err=0;

    theFileSelectionDialog->setFilterPattern("*.ppm");
    if(theFileSelectionDialog->postAndWait() == VkDialogManager::OK)
	err = morph::saveMovie(theFileSelectionDialog->fileName());

    if( err ) theErrorDialog->post(err);

    //---- End editable code block: mainWindow doSaveMovie

}    // End mainWindow::doSaveMovie()


void mainWindow::openFile ( Widget, XtPointer ) 
{
    //---- Start editable code block: mainWindow openFile

    // This virtual function is called from openFileCallback
    // Use the blocking mode of the file selection dialog
    // to get a file

    char *err=0;
    theFileSelectionDialog->setFilterPattern("*.cfg");
    if(theFileSelectionDialog->postAndWait() == VkDialogManager::OK)
        err = morph::open(theFileSelectionDialog->fileName());

    if( err ) theErrorDialog->post(err);

    //---- End editable code block: mainWindow openFile

}    // End mainWindow::openFile()

void mainWindow::quit ( Widget, XtPointer ) 
{
    // Exit via quitYourself() to allow the application
    // to go through its normal shutdown routines, checking with
    // all windows, views, and so on.

    theApplication->quitYourself();

}    // End mainWindow::quit()

void mainWindow::save ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: mainWindow save

    XmPushButtonCallbackStruct *cbs = (XmPushButtonCallbackStruct*) callData;

    if( morph::configFile[0]=='\0' ) saveas(w,callData);
    else morph::save(morph::configFile);

    //---- End editable code block: mainWindow save

}    // End mainWindow::save()


void mainWindow::saveas ( Widget, XtPointer ) 
{
    //---- Start editable code block: mainWindow saveas


    // This virtual function is called from saveasCallback
    // Use the blocking mode of the file selection dialog
    // to get a file

    theFileSelectionDialog->setFilterPattern("*.cfg");
    if(theFileSelectionDialog->postAndWait() == VkDialogManager::OK)
        morph::save(theFileSelectionDialog->fileName());

    //---- End editable code block: mainWindow saveas

}    // End mainWindow::saveAs()

void mainWindow::setInterpLines ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: mainWindow setInterpLines

    XmToggleButtonCallbackStruct *cbs = (XmToggleButtonCallbackStruct*) callData;
    morph::showInterpLines(cbs->set);

    //---- End editable code block: mainWindow setInterpLines

}    // End mainWindow::setInterpLines()


void mainWindow::setMorphed ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: mainWindow setMorphed

    morph::view = morph::morphed;

    //---- End editable code block: mainWindow setMorphed

}    // End mainWindow::setMorphed()


void mainWindow::setWarpedFinal ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: mainWindow setWarpedFinal

    morph::view = morph::warpedFinal;

    //---- End editable code block: mainWindow setWarpedFinal

}    // End mainWindow::setWarpedFinal()


void mainWindow::setWarpedInitial ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: mainWindow setWarpedInitial

    morph::view = morph::warpedInitial;

    //---- End editable code block: mainWindow setWarpedInitial

}    // End mainWindow::setWarpedInitial()





//---- Start editable code block: End of generated code



//---- End editable code block: End of generated code

