#ifndef __MORPH_H__
#define __MORPH_H__

#include <stdio.h>


typedef unsigned char uchar;

class morph {
public:
    static char configFile[100];
    
    typedef enum { warpedInitial, morphed, warpedFinal } t_view;
    static t_view view;

    static void showInterpLines(int val); // 0=off, 1 = on;
  
    // these functions get called by the UI when you select the respective menu
    // if they return a char*, is will be either NULL or an error message text
    static void init();
    static void nnew();
    static char* open(const char *file); //config file
    static char* save(const char *file);
    static char* loadInitialImg(const char *file); //rgb file
    static char* loadFinalImg(const char *file);
    static char* saveFrame(const char *file); //ppm file
    static char* saveMovie(const char *file); //ppm file

    // gets called when you move the slider
    static char* morphAtParameter(float t); // returns text with error(or NULL)

private:
    morph(){;} // should not be called. This class is just a wrapper for global variables
    static void directMap(float x, float y, int fromWin, int toWin,
			  float& xm, float &ym);
    static void inverseMap(float x, float y, int fromWin, int toWin,
			  float& xm, float &ym);


    static char initialImgName[100], finalImgName[100];
    static const unsigned char* initialImg, *finalImg;
    static unsigned char* morphedImg;
    static int width, height;


    ///////////////////////////////////////////////////////////////////////
    // The following functions are provided. You can use them to implement
    //the mouse user interface. The body of the functions is in InventorStuff.C
    // w represents the graphic window: 0=initial, 1= morph, 2=final
    // the image is texture-mapped onto a 1x1 square with low-left corner (0,0)
    // (just for rendering purposes it is scaled at the proper aspect ratio
    // when you call readImage_RGB)
    // so all the line endpoints coordinates are between 0..1
public:
    static int numLines(int w);
    static void newLine(int w);
    static void deleteLastLine(int w);
    static void deleteLine(int w,int n);
    static void setLine(int w, int n, float x1, float y1, float x2, float y2);
    static void getLine(int w, int n, float& x1, float& y1,
			float& x2, float& y2);
    static void setLineColor(int w, int n, uchar r, uchar g, uchar b);
    ///////////////////////////////////////////////////////////////////////


};

// These functions will be called by the Inventor code
// You must implement them. They should encapsulate the behavior of the mouse
// user interface (eg: make dragging the middle mouse button move line
// endpoints, 
// dragging the left mouse button add a new line,
// clicking the left button highlight the line pair,
// pressing DEL delete a line pair)

// button = mouse button (1/2)  (3 is reserved for Inventor pop-up menu)
// win = window(0=initial picture, 1=morphed picture, 2=final picture)
// (x,y) = image coordinates (between 0 and 1, origin = lower left)
//           might be (-1,-1) = invalid
// line = index of line segment closest to the point. may be -1 (=invalid)
// pt = index of closest endpoint (0/1) . may also be -1 (=invalid)

void mouseDown(int button,int win,
	       int winx, int winy, // pixels
	       float x, float y, // image coordinates 0..1
	       int line,int pt); 
void mouseUp(int button,int win, 
	     int winx, int winy,  // pixels
	     float x, float y, // image coordinates 0..1
	     int line,int pt); 
void mouseMove(int win,
	       int winx, int winy,  // pixels
	       float x, float y,
	       int line,int pt); 
void delKey(int win,int winx, int winy,float x, float y,int line,int pt);







#endif
